%*************************************************************%
%                        Sudoku-Lser                         %
%              Funktion zum Finden der Eckpunkte              %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser                                  %
%                                                             %
% Funktion: Die Eckpunkte des Sudokurahmens werden            %
%           identifiziert.                                    %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Toolbox:         Image Processing Toolbox                   %
%                                                             %
% Letzte nderung: 13.06.2015                                 %
%                                                             %
%*************************************************************%


function Eckpunkte = Eckpunkte_finden(SudokuBild)

%% Strungen im Originalbild zur Vorbereitung entfernen
% Originales Bild in ein binres Bild umwandeln
Schwellwert = graythresh(SudokuBild);
Binaerbild =~im2bw(SudokuBild,Schwellwert);

% Pixel am Rand entfernen
Sudoku_bereinigt = imclearborder(Binaerbild);

%% Sudukorahmen finden und Eckpunkte bestimmen
% Mithilfe der Funktion regionprops das grte Rechteck und damit die
% Rahmen des Rtsels bestimmen
BildEigenschaften = regionprops(Sudoku_bereinigt,'Area','BoundingBox','PixelList');

% Laufvariable fr die Suche des grten Rechtecks
Anzahl_Eigenschaften = numel(BildEigenschaften);

% Initialisieren der Variable max_Rechteck
max_Rechteck = 0;

% Anlegen des Vektors Rechteck_Flaeche, um darin alle mglichen groen
% Rechteckflchen zu hinterlegen
Rechteck_Flaeche = zeros(1,Anzahl_Eigenschaften);

% Grtes Rechteck suchen
for i = 1:Anzahl_Eigenschaften
    Rechteck_Flaeche(i) = prod(BildEigenschaften(i).BoundingBox(3:4));
    
    if BildEigenschaften(i).Area > max_Rechteck
        max_Rechteck = BildEigenschaften(i).Area;
        groesstes_Rechteck = i;
        
    end
end

% Berechnung der Diagonalen
% Diagonale von oben links nach unten rechts = lr
% Diagonale von oben rechts nach unten links = rl
Diagonale_lr = sum(BildEigenschaften(groesstes_Rechteck).PixelList,2);
Diagonale_rl = diff(BildEigenschaften(groesstes_Rechteck).PixelList,[],2);

% Eckpunkte: oben links = 1, oben rechts = 2, unten rechts = 3,
% unten links = 4

[~,Ecke_1] = min(Diagonale_lr);
[~,Ecke_2] = max(Diagonale_rl);
[~,Ecke_3] = max(Diagonale_lr);
[~,Ecke_4] = min(Diagonale_rl);


% Eckpunkte des Rechteckts speichern (vom Anfangspunkt ber die weiteren
% Eckpunkte bis zum Endpunkt (=Anfangspunkt)
Eckpunkte = BildEigenschaften(groesstes_Rechteck).PixelList([Ecke_1 Ecke_4 Ecke_3 Ecke_2 Ecke_1],:);

end % Funktionsende Eckpunkte_finden