%*************************************************************%
%                        Sudoku-Lser                         %
%             Funktion zum Finden der Kandidaten              %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser                                  %
%                                                             %
% Funktion: Das Sudoku-Rtsel sucht anhand der                %
%           identifizierten Ziffern mgliche Kandidaten der   %
%           leeren Zellen.                                    %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Toolbox:         Image Processing Toolbox                   %
%                                                             %
% Letzte nderung: 30.05.2015                                 %
%                                                             %
%*************************************************************%


function gefundene_Kandidaten = Kandidaten_finden(Sudoku)
%% Zelle mit allen mglichen Kandidaten schaffen
Kandidaten = cell(9,9);

%% Zu durchsuchende Zeilen, Spalten und Unterquadrate schaffen
[Zeile, Spalte] = find(Sudoku==0);
Unterquadrat    = cell({[1 2 3], [4 5 6], [7 8 9]});

%%  Mgliche Kandidaten anhand der identifizierten Ziffern suchen und in die Zelle eintragen
for i=1:length(find(Sudoku==0))
    
    % Mgliche Ziffern, die in ein Sudoku eingetragen werden knnen
    moegliche_Ziffern = 1:9;
    
    % Mgliche Kandiaten in der Zelle nach berprfung der Zeile und Spalte
    moegliche_Ziffern(nonzeros(Sudoku(Zeile(i), :))) = 0;
    moegliche_Ziffern(nonzeros(Sudoku(:, Spalte(i)))) = 0;
    
    % Zelle im Unterquadrat finden
    X_Koordinate = cellfun(@(x) any(find(x==Zeile(i))), Unterquadrat);
    Y_Koordinate = cellfun(@(x) any(find(x==Spalte(i))), Unterquadrat);
    
    Unterquadrat_Zeile  = Unterquadrat{X_Koordinate};
    Unterquadrat_Spalte = Unterquadrat{Y_Koordinate};
    
    % Unterquadrat aufspannen (als Matrix zwischenspeichern)
    Intervall_Zeile  = min(Unterquadrat_Zeile) : max(Unterquadrat_Zeile);
    Intervall_Spalte = min(Unterquadrat_Spalte) : max(Unterquadrat_Spalte);
    
    % Unterquadrat prfen
    moegliche_Ziffern(nonzeros(Sudoku(Intervall_Zeile, Intervall_Spalte))) = 0;
    
    % Mgliche Kandidaten ohne Nullen dargestellt
    Kandidaten{Zeile(i), Spalte(i)} = nonzeros(moegliche_Ziffern)';
    
end

gefundene_Kandidaten = Kandidaten;


end % Funktionsende Kandidaten_finden
