%*************************************************************%
%                       Sudoku-Lser                          %
%      Funktion zum Eintragen der Lsung in das Sudoku        %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser                                  %
%                                                             %
% Funktion: Die Lsung des Sudoku-Rtsels wird in das         %
%           entzerrte Rtsel in Matlab eingetragen.           %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Letzte nderung: 06.06.2015                                 %
%                                                             %
%*************************************************************%


function Sudoku_Loesung_eingetragen = Loesung_eintragen(Sudoku_entzerrt, Sudoku_Raetsel, Sudoku_geloest, Hoehe_Kaestchen, Breite_Kaestchen, SuchMatrix, Punkte)

%% Matrix erstellen, in der nur die Lsungswerte hinterlegt sind
Einzutragende_Loesung = Sudoku_geloest - Sudoku_Raetsel;

%% Lsungswerte in das entzerrte Bild eintragen

% Anlegen der Variablen, um zellenweise die Lsungen in die Matrix
% einzutragen
Hilfsvariable = 1;
Sudoku_Zeile  = 0;
Sudoku_Spalte = 0;

Sudoku_Loesung_eingetragen = Sudoku_entzerrt;

for i = 1:81
    
    if mod(i-1, 9)==0
        Sudoku_Zeile  = Sudoku_Zeile + Hilfsvariable; % eine Zeile weitergehen
        Sudoku_Spalte = Hilfsvariable;
    end
    
    % Abfrage, sodass nur Lsungswerte in das Bild eingetragen werden
    if Einzutragende_Loesung(Sudoku_Zeile, Sudoku_Spalte) ~=0
        
        % Lsungswert in String umwandeln
        LoesungsWert = Einzutragende_Loesung(Sudoku_Zeile, Sudoku_Spalte);
        str_LoesungsWert = num2str(LoesungsWert);
        
        % x- und y-Koordinate bestimmen, sodass die Lsung an die richtige
        % Stelle im Sudoku-Rtsel eingetragen wird
        x_Koordinate = round(SuchMatrix(i, 1) - Breite_Kaestchen*0.7 + Punkte(1,1));
        y_Koordinate = round(SuchMatrix(i, 2) - Hoehe_Kaestchen*0.8 + Punkte(1,2));
        
        % Lsungwerte in das Bild eintragen
        Loesung = vision.TextInserter(str_LoesungsWert);
        
        % Farbe des einzutragenden Lsungswertes bestimmen
        Loesung.Color = [0, 0, 255];
        
        % Schriftgre anhand der Kstchengre bestimmen
        Loesung.FontSize = round(Breite_Kaestchen/2);
        
        % Koordinaten der Lsungen hinterlegen
        Loesung.Location = [x_Koordinate, y_Koordinate];
        
        % Lsung in das Bild eintragen
        Sudoku_Loesung_eingetragen = step(Loesung, Sudoku_Loesung_eingetragen);
        
    end
    
    Sudoku_Spalte = Sudoku_Spalte + Hilfsvariable; % eine Spalte weitergehen
    
end

end % Funktionsende Loesung_eintragen