%*************************************************************%
%                        Sudoku-Lser                         %
%            Funktion zum Einlesen eines Sudokurtsels        %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser                                  %
%                                                             %
% Funktion: Das zu lsende klassische 9x9 Suduko-Rtsel wird  %
%           ber eine IP-Kamera oder aus einem Verzeichnis    % 
%           in Matlab eingelesen.                             %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Toolbox:         Image Processing Toolbox                   %
%                                                             %
% Letzte nderung: 18.06.2015                                 %
%                                                             %
%*************************************************************%


function Sudoku_eingelesen = Sudoku_einlesen
%% Abfrage, auf welche Art das Sudoku-Rtsel eingelesen wird
[Daten,Haken] = imread('Hilfsbilder/Haekchen.png'); % Hkchen fr die Messageboxes

Auswahl_Sudoku = questdlg('Whlen Sie bitte aus, wie das Sudoku-Rtsel eingelesen werden soll!', ...
    'Auswahl des Sudoku-Rtsels', 'ber eine IP-Kamera','Aus einem Verzeichnis','Abbrechen','Abbrechen');

% Switch-Case mit den Auswahlmglichkeiten zum Zuschneiden
switch Auswahl_Sudoku
    case 'ber eine IP-Kamera'
        waitfor(msgbox('Das Sudoku-Rtsel wird ber eine IP-Kamera fotografiert und eingelesen.',...
            'Erfolgreiche Auswahl','custom',Daten,Haken))
        Sudoku_Einleseart = 1;
    case 'Aus einem Verzeichnis'
        waitfor(msgbox([{'Das Sudoku-Rtsel wird aus einem Verzeichnis eingelesen.'},...
            {'Whlen Sie bitte das Sudoku-Rtsel aus dem Verzeichnis aus.'}],...
            'Erfolgreiche Auswahl', 'custom',Daten,Haken))
        Sudoku_Einleseart = 2;
    case 'Abbrechen'
        msgbox([{'Die Auswahl des Sudokus wurde abgebrochen.'},...
            {'Die Programmausfhrung wird beendet.'}], ...
            'Warnung: Abgebrochene Auswahl', 'warn')
        Sudoku_Einleseart = 3;
end

%% Sudoku-Rtsel fotografieren und einlesen
if Sudoku_Einleseart == 1
    % ber die IP-Kamera eines Smartphones wird ein Foto bei der Ausfhrung
    % dieses Skripts  aufgenommen und direkt eingelesen. Die wichtige
    % Information einer Aufnahme beinhaltet IP, Portnummer, Aufnahmemodus
    % und Bildformat.
    
    % Inputdialog zur Abfrage der IP-Adresse
    prompt = {'Geben Sie bitte die Adresse der IP-Kamera ein:'};
    dlg_title = 'Eingabe der Adresse';
    num_lines = 1;
    def = {'http://192.168.0.100:8080/shot.jpg','hsv'};
    Information= inputdlg(prompt,dlg_title,num_lines,def);
    
    url = char(Information(1,1));
    
    Sudoku_aufgenommen = Foto_aufnehmen(url);
    
    figure('NumberTitle', 'off', 'Name', 'Fotografiertes Sudoku');
    
    while 1
        
        imshow(Sudoku_aufgenommen)
        
        
        Auswahl_AufnahmeWiederholen = questdlg('Wenn Sie mit dem fotografierten Sudoku zufrieden sind, klicken Sie auf Fortfahren. Ansonsten knnen Sie erneut ein Foto aufnehmen oder das Programm abbrechen.', ...
            'Aufnahme des Sudoku-Rtsels', 'Fortfahren','Wiederholen','Abbrechen','Abbrechen');
        
        
        
        % Switch-Case mit der Auswahl, das Sudoku erneut zu fotografieren
        switch Auswahl_AufnahmeWiederholen
            case 'Fortfahren'
                
                AufnahmeWiederholen = 1;
            case 'Wiederholen'
                
                AufnahmeWiederholen = 2;
            case 'Abbrechen'
                msgbox([{'Die Auswahl der Zuschnittsart wurde abgebrochen.'},...
                    {'Die Programmausfhrung wird beendet.'}], ...
                    'Warnung: Abgebrochene Auswahl', 'warn')
                AufnahmeWiederholen = 3;
        end
        
        
        
        if AufnahmeWiederholen == 1;
            
            Sudoku_eingelesen = Sudoku_aufgenommen;
            break;
            
        elseif AufnahmeWiederholen == 2;
            
            Sudoku_aufgenommen = Foto_aufnehmen(url);
            
            
        elseif AufnahmeWiederholen == 3;
            
            error('Das Programm wurde beendet.')
            
        end
        
    end
    
end


%% Sudoku-Rtsel aus einem Verzeichnis laden
if Sudoku_Einleseart == 2
    [Bildname,Pfad] = uigetfile('*.jpg','Whlen Sie das Sudoku-Rtsel aus dem Verzeichnis aus');
    addpath(Pfad);
    Sudoku_eingelesen=imread(Bildname);
    figure('NumberTitle', 'off', 'Name', 'Original-Sudoku');
    imshow(Sudoku_eingelesen)
    
elseif Sudoku_Einleseart == 3
    
    return
    
end

end % Funktionsende Sudoku_einlesen