%*************************************************************%
%                       Sudoku-Solver                         %
% Funktion zur perspektivischen Entzerrung des Sudokurtsels  %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser                                  %
%                                                             %
% Funktion: Das eingelesene Suduko-Rtsel wird ber die       %
%           Ecken perspektivisch entzerrt.                    %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Toolbox:         Image Processing Toolbox                   %
%                                                             %
% Letzte nderung: 13.06.2015                                 %
%                                                             %
%*************************************************************%

function Sudoku_entzerrt = Sudoku_entzerren(Sudoku_eingelesen)

%% Abfrage, ob eine Entzerrung notwendig ist
Auswahl_Entzerrung = questdlg('Mchten Sie eine Entzerrung des Bildes durchfhren?', ...
    'Durchfhrung einer Entzerrung','Ja','Nein','Abbrechen','Abbrechen');

% Switch-Case mit den Auswahlmglichkeiten zum Entzerren
switch Auswahl_Entzerrung
    case 'Ja'
        Entzerrung = 1;
        
    case 'Nein'
        Entzerrung = 2;
        
    case 'Abbrechen'
        msgbox([{'Die Auswahl des Sudokus wurde abgebrochen.'},...
            {'Die Programmausfhrung wird beendet.'}], ...
            'Warnung: Abgebrochene Auswahl', 'warn')
        Entzerrung = 3;
end


%% Originales Bild entzerren
if Entzerrung == 1
    figure('NumberTitle', 'off', 'Name', 'Sudoku entzerren');
    imshow(Sudoku_eingelesen);
    
    % Punkte des verzerrten Rechtecks
    % Reihenfolge: linksoben, linksunten, rechtsunten, rechtsoben
    % (= gegen den Uhrzeigersinn)
    
    title('Klicken Sie auf alle vier Ecken des Sudoku-Rstels gegen Uhrzeigersinn.')
    [x,y] = ginput(4);
    Rechteck_verzerrt = [x,y];
    
    % Punkte des erwarteten Rechtecks (kleineres mgliches Rechteck whlen!)
    % Reihenfolge: linksoben, linksunten, rechtsunten, rechtsoben
    % (= gegen den Uhrzeigersinn)
    
    title('Klicken Sie nun auf alle vier Ecken des Sudoku-Rstels, die es nach der Entzerrung besitzen soll, gegen Uhrzeigersinn.')
    Rechteck_erwartet = ginput(4);
    
    % Transformation zur Entzerrung durchfhren
    Sudoku_erwartet = fitgeotrans(Rechteck_verzerrt,Rechteck_erwartet,'projective');
    Sudoku_transformiert = imwarp(Sudoku_eingelesen,Sudoku_erwartet);
    imshow(Sudoku_transformiert)
    
    
elseif Entzerrung == 2
    
    Sudoku_transformiert = Sudoku_eingelesen;
    
    
elseif Entzerrung == 3;
    
    error('Das Programm wurde beendet.')
    
end


%% Funktion zum Finden der Eckpunkte aufrufen
Eckpunkte = Eckpunkte_finden(Sudoku_transformiert);

%% Entzerrtes Bild rotieren

% Winkel mithilfe der oberen Eckpunkte berechnen
Gegenkathete = Eckpunkte(2,1) - Eckpunkte(1,1);
Ankathete    = Eckpunkte(1,2) - Eckpunkte(2,2);

Winkel = atand(Gegenkathete/Ankathete);

% Berechnung des Rotationswinkels
Rotationswinkel = Winkel - 90;

% Originales Bild um den Rotationswinkel entzerren
if Rotationswinkel > 1
    
    Sudoku_entzerrt = imrotate(Sudoku_transformiert, Rotationswinkel, 'bilinear', 'crop');
    
else
    
    Sudoku_entzerrt = Sudoku_transformiert;
    
end

figure('NumberTitle', 'off', 'Name', 'Entzerrtes Sudoku'); imshow(Sudoku_entzerrt);

end % Funktionsende Sudoku_entzerren