%*************************************************************%
%                        Sudoku-Lser                         %
%        Funktion zum Identifizieren des Sudokurtsels        %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser,                                 %
%           Idee der Templates(Buchstaben-/Ziffernerkennung)  %
%           basierend auf Diego Barragn Guerrero             %
%                                                             %
% Funktion: Die Ziffern des Suduko-Rtsels werden             %
%           identifiziert.                                    %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Toolbox:         Image Processing Toolbox                   %
%                                                             %
% Letzte nderung: 06.06.2015                                 %
%                                                             %
%*************************************************************%

function [Sudoku_Raetsel, Hoehe_Kaestchen, Breite_Kaestchen, SuchMatrix]  = Sudoku_identifizieren(Sudoku_zugeschnitten)

%% Templates in das Workspace laden
global Templates
Templates = Templates_erstellen;

%% Anzahl aller mglichen Ziffern eines Sudokus in einer Variablen speichern (1-9)
Ziffer_Moeglichkeiten = size(Templates,2);

%% Anlegen der Matrix, in der die x- und y-Koordinaten gespeichert werden
Sudoku_vorbereitet = Sudoku_zugeschnitten;

% Hhe und Breite eines einzelnen Kstchens bestimmen
Hoehe_Kaestchen  = round(size(Sudoku_zugeschnitten, 1)/9);
Breite_Kaestchen = round(size(Sudoku_zugeschnitten, 2)/9);

% Anlegen der Such-/Koordinatenmatrix
SuchMatrix = zeros(81,4);

% Initialisieren der bentigen Variablen
Hilfsvariable  = 1;
x_Koordinate   = 0;
y_Koordinate   = 0;
Sudoku_Spalte  = 0;
Sudoku_Zeile   = 0;

% Eintragen der Koordinaten in die Matrix
for i=1:length(SuchMatrix)
    
    if mod(i-1, 9)==0
        y_Koordinate  = y_Koordinate + Hoehe_Kaestchen;
        x_Koordinate  = Breite_Kaestchen;
        
        Sudoku_Zeile  = Sudoku_Zeile + Hilfsvariable; % eine Zeile weitergehen
        Sudoku_Spalte = Hilfsvariable;
    end
    
    SuchMatrix(i,1)   = x_Koordinate;
    SuchMatrix(i,2)   = y_Koordinate;
    
    SuchMatrix(i,3)   = Sudoku_Spalte;
    SuchMatrix(i,4)   = Sudoku_Zeile;
    
    x_Koordinate      = x_Koordinate + Breite_Kaestchen;
    Sudoku_Spalte     = Sudoku_Spalte + Hilfsvariable; % eine Spalte weitergehen
    
end

%% Anlegen der Matrix, in der die Ziffern des abfotografierten Sudoku eingetragen werden
Sudoku_Raetsel = zeros(9,9);

%% Vorbereiten des Bildes zur Identifizierung der Ziffern
% Bild in Grauskalenbild konvertieren, wenn dieses als RGB-Bild vorliegt
if size(Sudoku_vorbereitet,3)==3
    Sudoku_vorbereitet = rgb2gray(Sudoku_vorbereitet);
end

% Alle Objekte entfernen die kleiner 30 Pixel sind (Rauschentfernung)
Sudoku_vorbereitet = bwareaopen(Sudoku_vorbereitet,30);

% Neue Variable fr die Identifikation der Ziffern anlegen
Zeilen_verblieben  = Sudoku_vorbereitet;

%% Identifizieren der Ziffern sowie Speichern dieser in das Sudoku_Raetsel
Identifizierung = 1;

while Identifizierung
    % Sudoku zeilenweise lesen
    [Zeile1, Zeilen_verblieben] = Sudoku_lesen(Zeilen_verblieben);
    Sudoku_identifiziert = Zeile1;
    
    % Zusammenhngende Komponenten mit bwlabel im Sudoku finden
    [Label, Komponenten] = bwlabel(Sudoku_identifiziert);
    
    for k=2:Komponenten
        [Zeile,Spalte] = find(Label==k);
        
        % Ziffer extrahieren und diese auf Template-Gre skalieren
        Ziffer_erkannt=Sudoku_identifiziert(min(Zeile):max(Zeile),min(Spalte):max(Spalte));
        Ziffer_erkannt=imresize(Ziffer_erkannt,[42 24]);
        
        % Funktionsaufruf, die gefundenen Templates im Bild als Text zu speichern
        Zahl = Templates_konvertieren(Ziffer_erkannt,Ziffer_Moeglichkeiten);
        
        % Gefundene Werte ber die Koordinaten der SuchMatrix in das Sudoku_Raetsel eintragen
        for i=1:length(SuchMatrix)
            
            if SuchMatrix(i,1) > Spalte(1,1) && SuchMatrix(i,2) > Zeile(1,1)
                Sudoku_Raetsel(SuchMatrix(i,4), SuchMatrix(i,3)) = str2double(Zahl);
                break
            end
            
        end
        
    end
    
    % Ist die letzte Zeile leer, so wird die Schleife an dieser Stelle beendet
    if isempty(Zeilen_verblieben)
        break
    end
end

end % Funktionsende Sudoku_identifizieren