%*************************************************************%
%                        Sudoku-Lser                         %
%            Funktion zum Lsen des Sudokurtsels             %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser                                  %
%                                                             %
% Funktion: Das Sudoku-Rtsel wird anhand der identifizierten %
%           Ziffern gelst.                                   %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Toolbox:         Image Processing Toolbox                   %
%                                                             %
% Letzte nderung: 30.05.2015                                 %
%                                                             %
%*************************************************************%


function Sudoku = Sudoku_loesen(Sudoku)
%% Eindeutige Kandidaten werden im Sudoku eingesetzt
eindeutige_Kandidaten_vorhanden = 1;

while eindeutige_Kandidaten_vorhanden
    
    % Alle mglichen Kandidaten werden in einer cell gespeichert
    % (Funktionsaufrauf zum Finden der Kandidaten)
    gefundene_Kandidaten  = Kandidaten_finden(Sudoku);
    eindeutige_Kandidaten = find(Sudoku==0 & cellfun(@length, gefundene_Kandidaten)==1, 1);
    
    if (length(eindeutige_Kandidaten)) == 0
        
        break; % Sind alle eindeutigen Kandidaten eingesetzt, verlasse Schleife
        
    end
    
    % Setze eindeutige Kandidaten in das Sudoku ein
    Sudoku(eindeutige_Kandidaten) = gefundene_Kandidaten{eindeutige_Kandidaten};
    
end

% Sudoku wird zur berprfung auf Existenz dem Prfer bergeben
Sudoku_pruefen(Sudoku)

%% Rekursive Rckverfolgung, da keine eindeutigen Kandidaten mehr vorhanden sind
if (length(find(Sudoku==0))) > 0
    
    % Suche die erste leere Zelle
    moegliche_Ziffern = find(Sudoku(:) == 0,1);
    % Sudoku in eine Variable zwischenspeichern, falls eingesetzter Wert
    % zurckgesetzt werden muss, da dieser nicht zur Lsung fhrt
    Ursprung_Sudoku = Sudoku;
    
    % Die gefundenen Kandidaten werden in der Zelle solange nacheinander
    % ausprobiert, bis die Lsung eindeutig ist
    for i = [gefundene_Kandidaten{moegliche_Ziffern}]
        Sudoku = Ursprung_Sudoku;
        
        % Kandidaten einsetzen
        Sudoku(moegliche_Ziffern) = i;
        % Funktion zum Lesen aufrufen, um eine mgliche Lsung zu erhalten
        Sudoku = Sudoku_loesen(Sudoku);
        
        % Sind alle Zellen des Sudokus ausgefllt, ist das Sudoku gelst
        if length(nonzeros(Sudoku)) == 81
            return
        end
        
    end
    
end


end % Funktionsende Sudoku_loesen

