%*************************************************************%
%                        Sudoku-Lser                         %
%       Funktion zum Prfen des gelsten Sudokurtsels        %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser                                  %
%                                                             %
% Funktion: Das gelste Sudoku-Rtsel wird auf Existenz       %
%           geprft.                                          %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Toolbox:         Image Processing Toolbox                   %
%                                                             %
% Letzte nderung: 30.05.2015                                 %
%                                                             %
%*************************************************************%


function Sudoku_pruefen(Sudoku)
%% Zu durchsuchende Zeilen, Spalten und Unterquadrat(3x3) schaffen
[Zeile, Spalte] = find(Sudoku>0);
Unterquadrat = cell({[1 2 3], [4 5 6], [7 8 9]});

%% berprfung auf Existenz
for i=1:length(find(Sudoku>0))
    
    X_Koordinate = cellfun(@(x) any(find(x==Zeile(i))), Unterquadrat);
    Y_Koordinate = cellfun(@(x) any(find(x==Spalte(i))), Unterquadrat);
    
    Unterquadrat_Zeile  = Unterquadrat{X_Koordinate};
    Unterquadrat_Spalte = Unterquadrat{Y_Koordinate};
    
    Intervall_Zeile  = min(Unterquadrat_Zeile) : max(Unterquadrat_Zeile);
    Intervall_Spalte = min(Unterquadrat_Spalte) : max(Unterquadrat_Spalte);
    Unterquadrat_M   = Sudoku(Intervall_Zeile, Intervall_Spalte);
    
    % berprfe, ob in jeder Zeile, Spalte und in jedem Unterquadrat eine
    % Zahl mehr als einmal auftaucht: Wenn ja, ist das Sudoku nicht lsbar
    % und das Programm endet mit einem Error.
    
    for j=1:length(Sudoku)
        
        if(length(find(Sudoku(Zeile(i),:)==j)))>1 ||(length(find(Sudoku(:,Spalte(i))==j)))>1 ...
                ||(length(find(Unterquadrat_M==j)))>1
            
            errordlg('Das Sudoku ist nicht lsbar. Das Programm wird beendet! ',...
                'Sudoku nicht lsbar');
            error('Sudoku ist nicht lsbar')
            
        end
        
    end
    
end

end % Funktionsende Sudoku_pruefen

