%*************************************************************%
%                        Sudoku-Lser                         %
%    Funktion zum Zuschneiden des entzerrten Sudokurtsels    %
%*************************************************************%
%                                                             %
% Autor:    Anika Leinhuser                                  %
%                                                             %
% Funktion: Das Sudoku-Rtsel wird auf die Auenrahmen        %
%           zugeschnitten.                                    %
%                                                             %
%*************************************************************%
% Implementaion:   Matlab R2014a                              %
%                                                             %
% Toolbox:         Image Processing Toolbox                   %
%                                                             %
% Letzte nderung: 13.06.2015                                 %
%                                                             %
%*************************************************************%


function [Sudoku_zugeschnitten, Punkte] = Sudoku_zuschneiden(Sudoku_entzerrt)

%% Dialog zur Abfrage, wie das Sudoku-Rtsel zugeschnitten werden soll
[Daten,Haken] = imread('Hilfsbilder/Haekchen.png'); % Hkchen fr die Messageboxes

Auswahl_Zuschneiden = questdlg('Whlen Sie bitte die Zuschnittsart fr das Sudoku-Rtsel auf die Rahmenlinien aus!', ...
    'Auswahl der Zuschneidungsart', 'Manuell','Automatisch','Abbrechen','Abbrechen');

% Switch-Case mit den Auswahlmglichkeiten zum Zuschneiden
switch Auswahl_Zuschneiden
    case 'Manuell'
        waitfor(msgbox(['Die Zuschnittsart "' Auswahl_Zuschneiden '" wurde gewhlt.'],...
            'Erfolgreiche Auswahl','custom',Daten,Haken))
        Zuschnitt = 1;
    case 'Automatisch'
        waitfor(msgbox(['Die Zuschnittsart "' Auswahl_Zuschneiden '" wurde gewhlt.'],...
            'Erfolgreiche Auswahl', 'custom',Daten,Haken))
        Zuschnitt = 2;
    case 'Abbrechen'
        msgbox([{'Die Auswahl der Zuschnittsart wurde abgebrochen.'},...
            {'Die Programmausfhrung wird beendet.'}], ...
            'Warnung: Abgebrochene Auswahl', 'warn')
        Zuschnitt = 3;
end


%% Sudoku-Rtsel auf Rahmenlinien zuschneiden
if Zuschnitt == 1 % manuelles Zuschneiden
    
    figure('NumberTitle', 'off', 'Name', 'Zuschneiden des Sudoku'); imshow(Sudoku_entzerrt);
    
    % Obere linke Ecke auswhlen
    title(sprintf('Klicken Sie bitte auf die oberere linke Ecke des Sudoku-Rahmens.'));
    
    % Manuelles Auswhlen der oberen linken Ecke
    Punkte(1,:) = round(ginput(1));
    
    % Ecke einzeichnen
    hold on
    plot(Punkte(1,1),Punkte(1,2),'ro');
    text(Punkte(1,1)-15,Punkte(1,2)-10, 'linke obere Ecke', 'Color','r');
    
    % Untere rechte Ecke auswhlen
    title(sprintf('Klicken Sie bitte nun auf die untere rechte Ecke des Sudoku-Rahmens.'));
    
    % Manuelles Auswhlen der unteren rechten Ecke
    Punkte(2,:) = round(ginput(1)); %
    
    % Ecke einzeichnen
    hold on
    plot(Punkte(2,1),Punkte(2,2),'ro');
    text(Punkte(2,1)-20,Punkte(2,2)+15, 'untere rechte Ecke', 'Color','r');
    
    % Bild kurz pausieren, damit der Benutzer seine ausgewhlten Ecken
    % sieht
    pause(3);
    
    % Bild zuschneiden
    Sudoku_zugeschnitten = Sudoku_entzerrt(Punkte(1,2):Punkte(2,2),Punkte(1,1):Punkte(2,1)); % Zuschneiden auf die gewhlten Ecken
    
    % Sudoku in schwarz-weies (binres) Bild umwandeln
    Schwellwert          = graythresh(Sudoku_zugeschnitten);
    Sudoku_zugeschnitten =~im2bw(Sudoku_zugeschnitten,Schwellwert);
    
    
elseif Zuschnitt ==2 % automatisches Zuschneiden
    
    % Originales Sudoku schrittweise in ein schwarz-weies (binres) Bild umkehren
    Schwellwert_BW = graythresh(Sudoku_entzerrt);
    Sudoku_zugeschnitten =~ im2bw(Sudoku_entzerrt,Schwellwert_BW);
    
    % Weien Rand finden und darauf zuschneiden
    [Zeile, Spalte] = find(Sudoku_zugeschnitten==1);
    
    % X- und Y- Position der obere linken Ecke
    Start_x  = min(Spalte);
    Start_y  = min(Zeile);
    
    % oberen Eckpunkt fr das sptere Eintragen der Lsung speichern
    Punkte = [Start_x, Start_y];
    
    % X- und Y- Position der unteren rechten Ecke
    Ende_x = max(Spalte);
    Ende_y = max(Zeile);
    
    % Differenz der beiden Punkte bestimmen
    Diff_x = Ende_x - Start_x;
    Diff_y = Ende_y - Start_y;
    
    % Auf die beiden Eckpunkte das orginale Sudoku zuschneiden
    Sudoku_zugeschnitten = imcrop(Sudoku_zugeschnitten,[Start_x Start_y Diff_x Diff_y]);
    
else
    return
end

% Zugeschnittenes Sudoku anzeigen
figure('NumberTitle', 'off', 'Name', 'Zugeschnittenes Sudoku'), imshow(Sudoku_zugeschnitten);

end % Funktionsende Sudoku_zuschneiden